<?php

if (!defined('_PS_VERSION_'))
	exit;
	
class cash24 extends PaymentModule
{
	public $host_url;
	
	public function __construct()
	{
		$this->name = 'cash24';
		$this->tab = 'payments_gateways';
		$this->version = '1.0';
		$this->author = 'Cash24';
		$this->controllers = array('payment', 'validation');

//		$this->bootstrap = true;
		parent::__construct();

		$this->displayName = 'Cash24';
		$this->description = $this->l('Receive payment with Cash24');

		$this->host_url = (Configuration::get('PS_SHOP_DOMAIN') ? Configuration::get('PS_SHOP_DOMAIN') : $_SERVER['HTTP_HOST']);

		$this->_payment_systems = array(
			''	=> array( 'title' => $this->l('Универсальная интернет-касса Cash24'), 'unitId' => 0, 'accountId' => 0, 'isInvoice' => 0 ),
			'alfa-click'	=> array( 'title' => $this->l('АльфаБанк, AlfaClick (Cash24)'), 'unitId' => 587412, 'accountId' => 0, 'isInvoice' => 0 ),
			'card'	=> array( 'title' => $this->l('Банковская карта (Cash24)'), 'unitId' => 545234, 'accountId' => 0, 'isInvoice' => 0 ),
			'card-beeline'	=> array( 'title' => $this->l('Со счета мобильного Билайн (Cash24)'), 'unitId' => 609111, 'accountId' => 0, 'isInvoice' => 0 ),
			'card-megafon'	=> array( 'title' => $this->l('Со счета мобильного Мегафон (Cash24)'), 'unitId' => 1015, 'accountId' => 94, 'isInvoice' => 0 ),
			'qiwi'	=> array( 'title' => $this->l('QIWI (Cash24)'), 'unitId' => 499669, 'accountId' => 159, 'isInvoice' => 0 ),
			'wmz'		=> array( 'title' => $this->l('Webmoney, WMZ (Cash24)'), 'unitId' => 661709, 'accountId' => 0, 'isInvoice' => 0 ),
			'wmr'	=> array( 'title' => $this->l('Webmoney, WMR (Cash24)'), 'unitId' => 1017, 'isInvoice' => 0 ),
			'wmu'	=> array( 'title' => $this->l('Webmoney, WMU (Cash24)'), 'unitId' => 1020, 'accountId' => 16, 'isInvoice' => 0 ),
			'wme'		=> array( 'title' => $this->l('Webmoney, WME (Cash24)'), 'unitId' => 822360, 'accountId' => 0, 'isInvoice' => 0 ),
			'yandex'	=> array( 'title' => $this->l('Яндекс деньги (Cash24)'), 'unitId' => 705000, 'accountId' => 75983431, 'isInvoice' => 1 ),
		);
	
		
	}

	public function install()
	{
		if (!parent::install() OR 
			!$this->registerHook('payment') OR 
			!$this->registerHook('orderConfirmation'))
			return false;
		Configuration::updateValue('CASH24_MERCHANT_ID', '');
		Configuration::updateValue('CASH24_SECRET_KEY', '');
		Configuration::updateValue('CASH24_COMMAND_KEY', '');
		Configuration::updateValue('CASH24_TEST_MODE', '0');
		Configuration::updateValue('CASH24_EXP', '3');
		Configuration::updateValue('CASH24_PAYMENT_SYSTEMS', '');

		$this->createCashPaymentStatus('CASH24_OS_WAITING', 'Ожидание Cash24', '#3333FF', 'payment_waiting2', false, false, '', false);
		$this->createCashPaymentStatus('CASH24_OS_HOLD', 'Выполнен Холд Cash24', '#32cd32', 'payment_hold', true, true, true, true);
$this->createCashPaymentStatus('CASH24_OS_CANCEL', 'Счет отменен Cash24', '#32cd32', 'payment_canc', true, true, true, true);
$this->createCashPaymentStatus('CASH24_OS_PAID', 'Счет оплачен Cash24', '#32cd32', 'payment_paid', true, true, true, true);
$this->createCashPaymentStatus('CASH24_OS_REFUND', 'Выполнен возврат Cash24', '#32cd32', 'payment_refund', true, true, true, true);
		return true;
	}

	/**
	 * create new order statuses
	 *
	 * @param $string
	 * @param $string
	 * @param $color
	 * @param $template
	 * @param $invoice
	 * @param $send_email
	 * @param $paid
	 * @param $logable
	 */
	public function createCashPaymentStatus($status, $title, $color, $template, $invoice, $send_email, $paid, $logable)
	{
		$ow_status = Configuration::get($status);
		if ($ow_status === false)
		{
			$order_state = new OrderState();
			//$order_state->id_order_state = (int)$key;
		}
		else
			$order_state = new OrderState((int)$ow_status);

		$langs = Language::getLanguages();

		foreach ($langs as $lang)
			$order_state->name[$lang['id_lang']] = $title;
         
		$order_state->invoice = $invoice;
		$order_state->send_email = $send_email;

		if ($template != '')
			$order_state->template = $template;

		if ($paid != '')
			$order_state->paid = $paid;

		$order_state->logable = $logable;
		$order_state->color = $color;
		$order_state->save();

		Configuration::updateValue($status, (int)$order_state->id);

		copy(dirname(__FILE__).'/bag_cash24.png', dirname(__FILE__).'/../../img/os/'.(int)$order_state->id.'.gif');
	}

	public function uninstall()
	{
		Configuration::deleteByName('CASH24_MERCHANT_ID');
		Configuration::deleteByName('CASH24_SECRET_KEY');
		Configuration::deleteByName('CASH24_COMMAND_KEY');
		Configuration::deleteByName('CASH24_TEST_MODE');
		Configuration::deleteByName('CASH24_EXP');
		Configuration::deleteByName('CASH24_PAYMENT_SYSTEMS');


		return parent::uninstall();
	}

	public function hookOrderConfirmation($params)
	{
		global $smarty;

		if ($params['objOrder']->module != $this->name) 
			return;

		$status = Tools::getvalue('status');
		if ($status == 'success')
		{
			if ($params['objOrder']->getCurrentState() != Configuration::get('PAYANYWAY_OS_SUCCEED'))
				$smarty->assign(array('status' => 'waiting', 'id_order' => intval($params['objOrder']->id)));
			else
				$smarty->assign(array('status' => 'success', 'id_order' => intval($params['objOrder']->id)));
		}
		elseif ($status == 'fail')
		{
			$params['objOrder']->setCurrentState(Configuration::get('PS_OS_CANCELED'));
			$smarty->assign('status', 'failed');
		}
		elseif ($status == 'invoice')
		{
			$smarty->assign(array('status' => 'invoice', 'invoice' => $this->getInvoice($params)));
		}

		return $this->display(__FILE__, 'hookorderconfirmation.tpl');
	}
	
	private function getInvoice($params)
	{
		global $smarty;
		
		$payanyway_server = Configuration::get('PAYANYWAY_PAYMENT_URL');
		$login = Configuration::get('PAYANYWAY_LOGIN');
		$password = Configuration::get('PAYANYWAY_PASSWORD');

		require_once(dirname(__FILE__) . '/MonetaAPI/MonetaWebService.php');
		switch ($payanyway_server) {
			case 'www.payanyway.ru':
				$service = new MonetaWebService("https://www.moneta.ru/services.wsdl", $login, $password);
				break;
			case 'demo.moneta.ru':
				$service = new MonetaWebService("https://demo.moneta.ru/services.wsdl", $login, $password);
				break;
		}

		try
		{
			$totalAmount = Tools::getvalue('MNT_AMOUNT')." ".Tools::getvalue('MNT_CURRENCY_CODE');
			$fee = "-";

			// запрос стоимости и комиссии
			$accountId = Tools::getvalue('paymentSystem_accountId', null);
			if (!is_null($accountId)) {
				$transactionRequestType = new MonetaForecastTransactionRequest();
				$transactionRequestType->payer = Tools::getvalue('paymentSystem_accountId');
				$transactionRequestType->payee = Tools::getvalue('MNT_ID');
				$transactionRequestType->amount = Tools::getvalue('MNT_AMOUNT');
				$transactionRequestType->clientTransaction = Tools::getvalue('MNT_TRANSACTION_ID');
				$forecast = $service->ForecastTransaction($transactionRequestType);
				$totalAmount = number_format($forecast->payerAmount,2,'.','')." ".$forecast->payerCurrency;
				$fee = number_format($forecast->payerFee,2,'.','')." ".$forecast->payerCurrency;
			}
			
			// получить данные счета
			$request = new MonetaInvoiceRequest();
			if (!is_null($accountId))
				$request->payer = $accountId;
			$request->payee = Tools::getvalue('MNT_ID');
			$request->amount = Tools::getvalue('MNT_AMOUNT');
			$request->clientTransaction = Tools::getvalue('MNT_TRANSACTION_ID');

			$operationInfo = new MonetaOperationInfo();
			if (Tools::getvalue('payment_system') == 'post')
			{
				$a = new MonetaKeyValueAttribute();
				$a->key = 'mailofrussiaindex';
				$a->value = Tools::getvalue('additionalParameters_mailofrussiaSenderIndex');
				$operationInfo->addAttribute($a);
				$a1 = new MonetaKeyValueAttribute();
				$a1->key = 'mailofrussiaindex';
				$a1->value = Tools::getvalue('additionalParameters_mailofrussiaSenderRegion');
				$operationInfo->addAttribute($a1);
				$a2 = new MonetaKeyValueAttribute();
				$a2->key = 'mailofrussiaaddress';
				$a2->value = Tools::getvalue('additionalParameters_mailofrussiaSenderAddress');
				$operationInfo->addAttribute($a2);
				$a3 = new MonetaKeyValueAttribute();
				$a3->key = 'mailofrussianame';
				$a3->value = Tools::getvalue('additionalParameters_mailofrussiaSenderName');
				$operationInfo->addAttribute($a3);
			}
			elseif (Tools::getvalue('payment_system') == 'svyaznoi')
			{
				$a1 = new MonetaKeyValueAttribute();
				$a1->key = 'rapidamphone';
				$a1->value = Tools::getvalue('additionalParameters_rapidaPhone');
				$operationInfo->addAttribute($a1);
			}
			$request->operationInfo = $operationInfo;
			
			$response = $service->Invoice($request);
			if (Tools::getvalue('payment_system') == 'svyaznoi')
			{
				$response1 = $service->GetOperationDetailsById($response->transaction);
				foreach ($response1->operation->attribute as $attr)
				{
					if ($attr->key == 'rapidatid')
					{
						$transaction_id = $attr->value;
					}
				}
			}
			else
			{
				$transaction_id = $response->transaction;
			}

			$invoice = array();
			$invoice['status'] = $response->status;
			$invoice['system'] = Tools::getvalue('payment_system');
			$invoice['payanyway_server'] = $payanyway_server;
			$invoice['transaction'] = str_pad($transaction_id, 10, "0", STR_PAD_LEFT);
			$invoice['ctid'] = Tools::getvalue('MNT_TRANSACTION_ID');
			$invoice['operation'] = $response->transaction;
			$invoice['unitId'] = Tools::getvalue('paymentSystem_unitId');
			$invoice['amount'] = $totalAmount;
			$invoice['payerAmount'] = $totalAmount;
			$invoice['fee'] = $fee;
		}
		catch (Exception $e)
		{
			$invoice['status'] = 'FAILED';
			$invoice['error_message'] = $e->getMessage();
		}
		return $invoice;
	}
	
	public function getContent()
	{
		$output = '<h2>'.$this->displayName.'</h2>';
		
		if (Tools::isSubmit('submitModule'))
		{
			
			Configuration::updateValue('CASH24_MERCHANT_ID', Tools::getvalue('CASH24_MERCHANT_ID'));
			Configuration::updateValue('CASH24_SECRET_KEY', Tools::getvalue('CASH24_SECRET_KEY'));
			Configuration::updateValue('CASH24_COMMAND_KEY', Tools::getvalue('CASH24_COMMAND_KEY'));
			Configuration::updateValue('CASH24_EXP', Tools::getvalue('CASH24_EXP'));
			Configuration::updateValue('CASH24_TEST_MODE', Tools::getvalue('CASH24_TEST_MODE'));
			Configuration::updateValue('CASH24_PAYMENT_SYSTEMS', serialize(Tools::getvalue('payment_systems')));

			$output .= $this->displayConfirmation($this->l('Configuration updated'));
		}

		$output .= '
		<fieldset>
			<a href="http://www.cash24.ru" target="_blank"  style="float:left;margin-right:15px;"><img src="../modules/cash24/logo.png" alt="PayAnyWay" /></a>
			
			<br />
		</fieldset><br />
		<form action="'.Tools::htmlentitiesutf8($_SERVER['REQUEST_URI']).'" method="post">
			<fieldset>
				<legend><img src="../img/admin/contact.gif" alt="" />'.$this->l('Settings').'</legend>
				
				<label for="CASH24_MERCHANT_ID">'.$this->l('Merchant Id').'</label>
				<div class="margin-form"><input type="text" size="20" id="CASH24_MERCHANT_ID" name="CASH24_MERCHANT_ID" value="'.Configuration::get('CASH24_MERCHANT_ID').'" /></div>
				<label for="CASH24_SECRET_KEY">'.$this->l('Secret Key').'</label>
				<div class="margin-form"><input type="text" size="20" id="CASH24_SECRET_KEY" name="CASH24_SECRET_KEY" value="'.Configuration::get('CASH24_SECRET_KEY').'" /></div>
				<label for="CASH24_COMMAND_KEY">'.$this->l('Command Key').'</label>
				<div class="margin-form"><input type="text" size="20" id="CASH24_COMMAND_KEY" name="CASH24_COMMAND_KEY" value="'.Configuration::get('CASH24_COMMAND_KEY').'" /></div>
				<label for="CASH24_EXP">'.$this->l('Срок действия счета (суток)').'</label>
				<div class="margin-form"><input type="text" size="20" id="CASH24_EXP" name="CASH24_EXP" value="'.Configuration::get('CASH24_EXP').'" /></div>
				<label for="CASH24_TEST_MODE">'.$this->l('Режим').'</label>

				<div class="margin-form" id="CASH24_TEST_MODE">
					<input type="radio" name="CASH24_TEST_MODE" value="0" style="vertical-align: middle;" '.(!Tools::getValue('CASH24_TEST_MODE', Configuration::get('CASH24_TEST_MODE')) ? 'checked="checked"' : '').' />
					<span style="color: #080;">'.$this->l('Рабочий').'</span>
					<input type="radio" name="CASH24_TEST_MODE" value="1" style="vertical-align: middle;" '.(Tools::getValue('CASH24_TEST_MODE', Configuration::get('CASH24_TEST_MODE')) ? 'checked="checked"' : '').' />
					<span style="color: #900;">'.$this->l('Тестовый').'</span>
				</div>
				
				<label for="cash24_systems">'.$this->l('Payment systems').'</label>
				<div class="margin-form" id="cash24_systems">';
			
			$payment_systems = unserialize(Configuration::get('CASH24_PAYMENT_SYSTEMS'));
			$payment_systems = is_array($payment_systems) ? $payment_systems : array();
			foreach ($this->_payment_systems as $key => $value)
			{
				$output .= '<input type="checkbox" id="'.$key.'" name="payment_systems[]" value="'.$key.'" '.((in_array($key, $payment_systems)) ? 'checked="checked"' : '').' />
							<span>'.$value['title'].'</span><br />';
			}
			$output .=	'
				</div>
				
				<br /><center><input type="submit" name="submitModule" value="'.$this->l('Сохранить').'" class="button" /></center>
			</fieldset>
		</form>';
		
		return $output;
	}

	public function hookPayment($params)
	{
		if (!$this->active)
			return ;
		if (!$this->checkCurrency($params['cart']))
			return ;
		
		global $cookie, $smarty;
		
		$payment_systems = unserialize(Configuration::get('CASH24_PAYMENT_SYSTEMS'));
		$payment_systems = is_array($payment_systems) ? $payment_systems : array();
		$payment_systems = array_intersect_key($this->_payment_systems,array_flip($payment_systems));

		$smarty->assign('payment_systems', $payment_systems);
		
		return $this->display(__FILE__, 'payment.tpl');
	}
	
	public function checkCurrency($cart)
	{
		$currency_order = new Currency((int)($cart->id_currency));
		$currencies_module = $this->getCurrency((int)$cart->id_currency);
		$currency_default = Configuration::get('PS_CURRENCY_DEFAULT');
		
		if (is_array($currencies_module))
			foreach ($currencies_module AS $currency_module)
				if ($currency_order->id == $currency_module['id_currency'])
					return true;
		return false;
	}
}
?>
